"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _eventstoreBase = require("@resolve-js/eventstore-base");

const getCursorUntilEventTypes = async ({
  database,
  escapeId,
  escape,
  eventsTableName,
  shapeEvent
}, cursor, untilEventTypes) => {
  if (untilEventTypes.length < 1) {
    throw new Error('Must define at least one event type');
  }

  const tableNameAsId = escapeId(eventsTableName);
  const vectorConditions = (0, _eventstoreBase.cursorToThreadArray)(cursor);
  const minThreadCounterConditions = `${vectorConditions.map((threadCounter, threadId) => `${escapeId('threadId')} = ${+threadId} AND ${escapeId('threadCounter')} >= ${+threadCounter} `).join(' OR ')}`;
  const rows = await database.all(`SELECT "threadId", MIN("threadCounter") AS "threadCounter" FROM (
          SELECT "threadId", MIN("threadCounter") AS "threadCounter" FROM ${tableNameAsId} WHERE type IN 
          (${untilEventTypes.map(t => escape(t)).join(', ')}) 
          AND (${minThreadCounterConditions}) 
          GROUP BY "threadId"
          UNION ALL
          SELECT "threadId", MAX("threadCounter")+1 AS "threadCounter" FROM ${tableNameAsId} GROUP BY "threadId")
        GROUP BY "threadId"`);
  const threadCounters = (0, _eventstoreBase.initThreadArray)();

  for (const row of rows) {
    threadCounters[row.threadId] = +row.threadCounter;
  }

  return (0, _eventstoreBase.threadArrayToCursor)(threadCounters);
};

var _default = getCursorUntilEventTypes;
exports.default = _default;
//# sourceMappingURL=get-cursor-until-event-types.js.map